/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.platform.raspberry.generator.extension;

import org.fortiss.af3.component.model.OutputPort;
import org.fortiss.af3.platform.raspberry.generator.extension.library.PiHALLibraryGeneratorExtensionBase;
import org.fortiss.af3.platform.raspberry.generator.framework.ISingletonGeneratorExtension;
import org.fortiss.af3.platform.raspberry.generator.framework.IWriteableGeneratorExtension;
import org.fortiss.af3.platform.raspberry.model.ActuatorPWM;

/** Executable for {@link ActuatorPWM}. */
public class PWMActuatorGeneratorExtension extends PiHALLibraryGeneratorExtensionBase<ActuatorPWM>
		implements ISingletonGeneratorExtension, IWriteableGeneratorExtension {
	/** Constructor. */
	public PWMActuatorGeneratorExtension(ActuatorPWM modelElement) {
		super(modelElement);
	}

	/** {@inheritDoc} */
	@Override
	public String getHeaderFileName() {
		return "temp_actuator.h";
	}

	/** {@inheritDoc} */
	@Override
	public String getSingletonIdentifier() {
		// Using header file as unique ID
		return getHeaderFileName();
	}

	/** {@inheritDoc} */
	@Override
	public String getSingletonVariableDeclarationCode() {
		return "static int actuator_pwm_" + getSingletonPostfix() + " = -1;\n";
	}

	/** {@inheritDoc} */
	@Override
	public String getSingletonAuxiliaryFunctions() {
		return null;
	}

	/** {@inheritDoc} */
	@Override
	public String getSingletonInitializationCode() {
		return "actuator_pwm_" + getSingletonPostfix() +
				" = temp_actuator_initialize(\"/dev/ttyACM0\");\n";
	}

	/** {@inheritDoc} */
	@Override
	public String getSingletonTerminationCode() {
		return "temp_actuator_terminate();\n";
	}

	/** {@inheritDoc} */
	@Override
	public String getWriteCode(OutputPort logicalSignal, String value) {
		return "temp_actuator_device_set_target(actuator_pwm_" + getSingletonPostfix() + ", " +
				modelElement.getChannelID() + ", " + value + ");\n";
	}

	/** {@inheritDoc} */
	@Override
	public String getNoValWriteCode(OutputPort logicalSignal) {
		return null;
	}
}
